@@ -1,13 +1,15 @@
 # -*- coding: utf-8 -*-
-from .fast_ast import FastAst
-from .misc_ast_utils import (
+from pyccolo._fast.fast_ast import FastAst
+from pyccolo._fast.misc_ast_utils import (
     EmitterMixin,
+    copy_ast,
     make_composite_condition,
     make_test,
     subscript_to_slice,
 )
 
 __all__ = [
+    "copy_ast",
     "EmitterMixin",
     "FastAst",
     "make_composite_condition",
