@@ -63,6 +63,9 @@ Skip = object()
 SANDBOX_FNAME = "<sandbox>"
 
 
+PYCCOLO_DEV_MODE_ENV_VAR = "PYCCOLO_DEV_MODE"
+
+
 def register_tracer_state_machine(tracer_cls: "Type[BaseTracer]") -> None:
     tracer_cls.EVENT_HANDLERS_BY_CLASS[tracer_cls] = defaultdict(
         list, tracer_cls.EVENT_HANDLERS_PENDING_REGISTRATION
@@ -131,6 +134,7 @@ class _InternalBaseTracer(metaclass=MetaTracerStateMachine):
     current_module: List[Optional[ast.Module]] = [None]
 
     def __init__(self, is_reset: bool = False):
+        self._is_dev_mode = os.environ.get(PYCCOLO_DEV_MODE_ENV_VAR) == "1"
         if is_reset:
             return
         if not self._MANAGER_CLASS_REGISTERED:
@@ -339,7 +343,7 @@ class _InternalBaseTracer(metaclass=MetaTracerStateMachine):
                 except Exception as exc:
                     if self.should_propagate_handler_exception(event, exc):
                         raise exc
-                    else:
+                    elif self._is_dev_mode:
                         logger.exception("An exception while handling evt %s", event)
                     new_ret = None
                 if new_ret is SkipAll:
@@ -401,18 +405,23 @@ class _InternalBaseTracer(metaclass=MetaTracerStateMachine):
             assert self._is_tracing_enabled
             assert not has_sys_trace_events or sys.gettrace() is self.sys_tracer
         self._is_tracing_enabled = False
-        if has_sys_trace_events:
+        if has_sys_trace_events and sys.gettrace() is not None:
             sys_settrace(self.existing_tracer)
         if len(_TRACER_STACK) == 0:
             setattr(builtins, TRACING_ENABLED, False)
 
     def _patch_sys_settrace_non_context(self) -> Callable:
+        import threading
+
         original_sys_settrace = sys.settrace
+        orig_thread = threading.current_thread()
 
         def cleanup_callback():
             sys.settrace = original_sys_settrace
 
         def patched_sys_settrace(trace_func):  # pragma: no cover
+            if threading.current_thread() is not orig_thread:
+                return original_sys_settrace(trace_func)
             # called by third-party tracers
             self.existing_tracer = trace_func
             if self._is_tracing_enabled:
@@ -808,7 +817,7 @@ class _InternalBaseTracer(metaclass=MetaTracerStateMachine):
             )
         return local_env.pop(env_name)
 
-    def trace_lambda(self, lam):
+    def trace_lambda(self, lam: Callable[..., Any]) -> Callable[..., Any]:
         # for now, this is primarily so that we can distinguish between
         # lambdas that we generate vs that user generates
         code: CodeType = lam.__code__
@@ -955,7 +964,7 @@ def register_universal_handler(handler):
 
 
 class BaseTracer(_InternalBaseTracer):
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args, **kwargs) -> None:
         super().__init__(*args, **kwargs)
         self._saved_slice: Optional[Any] = None
 
